#pragma once
#include <gio/gio.h>
typedef GError RORGError;
typedef GHashTable RORGHashTable;
typedef GPtrArray RORGPtrArray;

/* Warning, this file is autogenerated by cbindgen. Don't modify this manually. */

#include <stdarg.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>

#define RORINCLUDE_MAXDEPTH 50

/**
 * Context object used to iterate through `HistoryEntry` events.
 */
typedef struct RORHistoryCtx RORHistoryCtx;

typedef struct RORTreefile RORTreefile;

/**
 * A history entry in the journal. It may represent multiple consecutive boots
 * into the same deployment. This struct is exposed directly via FFI to C.
 */
typedef struct {
  /**
   * The deployment root timestamp.
   */
  uint64_t deploy_timestamp;
  /**
   * The command-line that was used to create the deployment, if any.
   */
  char *deploy_cmdline;
  /**
   * The number of consecutive times the deployment was booted.
   */
  uint64_t boot_count;
  /**
   * The first time the deployment was booted if multiple consecutive times.
   */
  uint64_t first_boot_timestamp;
  /**
   * The last time the deployment was booted if multiple consecutive times.
   */
  uint64_t last_boot_timestamp;
  /**
   * `true` if there are no more entries.
   */
  bool eof;
} RORHistoryEntry;

int ror_compose_postprocess_final(int rootfs_dfd, RORGError **gerror);

int ror_download_to_fd(const char *url, RORGError **gerror);

void ror_history_ctx_free(RORHistoryCtx *hist);

RORHistoryCtx *ror_history_ctx_new(RORGError **gerror);

int ror_history_ctx_next(RORHistoryCtx *hist, RORHistoryEntry *entry, RORGError **gerror);

void ror_history_entry_clear(RORHistoryEntry *entry);

int ror_history_prune(RORGError **gerror);

void ror_init_logging(void);

int ror_journal_print_staging_failure(RORGError **gerror);

RORGHashTable *ror_lockfile_read(char **filenames, RORGError **gerror);

int ror_lockfile_write(const char *filename, RORGPtrArray *packages, RORGError **gerror);

void ror_progress_begin_n_items(const char *msg, int n);

void ror_progress_begin_percent(const char *msg);

void ror_progress_begin_task(const char *msg);

void ror_progress_end(const char *suffix);

void ror_progress_set_message(const char *msg);

void ror_progress_set_sub_message(const char *msg);

void ror_progress_update(int n);

void ror_treefile_free(RORTreefile *tf);

int ror_treefile_get_add_file_fd(RORTreefile *tf, const char *filename);

char **ror_treefile_get_all_ostree_layers(RORTreefile *tf);

const char *ror_treefile_get_checksum(RORTreefile *tf);

int ror_treefile_get_dfd(RORTreefile *tf);

int ror_treefile_get_group_fd(RORTreefile *tf);

const char *ror_treefile_get_json_string(RORTreefile *tf);

char **ror_treefile_get_ostree_layers(RORTreefile *tf);

char **ror_treefile_get_ostree_override_layers(RORTreefile *tf);

int ror_treefile_get_passwd_fd(RORTreefile *tf);

int ror_treefile_get_postprocess_script_fd(RORTreefile *tf);

const char *ror_treefile_get_rojig_name(RORTreefile *tf);

const char *ror_treefile_get_rojig_spec_path(RORTreefile *tf);

RORTreefile *ror_treefile_new(const char *filename,
                              const char *basearch,
                              int workdir_dfd,
                              RORGError **gerror);

char *ror_util_varsubst(const char *s, RORGHashTable *h, RORGError **gerror);

G_DEFINE_AUTOPTR_CLEANUP_FUNC(RORTreefile, ror_treefile_free)
G_DEFINE_AUTOPTR_CLEANUP_FUNC(RORHistoryCtx, ror_history_ctx_free)
G_DEFINE_AUTO_CLEANUP_CLEAR_FUNC(RORHistoryEntry, ror_history_entry_clear)

